<?php

/**
 * @Copyright Copyright (C) 2021 jtotal.org All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 * @link https://jtotal.org
 */

// no direct access
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Profiler\Profiler;
use Joomla\CMS\Uri\Uri;

use Joomla\CMS\Helper\ModuleHelper;
use Joomla\CMS\Version;
use Joomla\Module\VisualMenu\Site\Helper\VisualMenuHelper;
use Joomla\Module\VisualMenu\Site\Helper\VisualMenuDebug;

require_once __DIR__ . '/src/Helper/VisualMenuHelper.php';
require_once __DIR__ . '/src/Helper/VisualMenuDebug.php';
require_once __DIR__ . '/src/Helper/VisualMenuImageManager.php';

/** @var Joomla\CMS\WebAsset\WebAssetManager $wa */
$wa = $app->getDocument()->getWebAssetManager();

// Enable assets
$wr = $app->getDocument()->getWebAssetManager()->getRegistry();
$wr->addRegistryFile('media/mod_visual_menu/joomla.asset.json');


$debug = $params->get('visual_menu_debug', 0);

if ($debug) {
    $profiler = new Profiler();
}

$jinput = Factory::getApplication()->input;
$document = $app->getDocument();


$show_readmore = $params->get('show_readmore', 0);
$show_readmore_target = $params->get('show_readmore_target', '_self');
$readmore_class = $params->get('readmore_class', 'btn btn-success');

$vm_titles_h = $params->get('vm_titles_h', '0');
$use_article_url = $params->get('use_article_url', 0);

$img_position = $params->get('img_position', 'top');
$img_opacity = $params->get('img_overlay_opacity', '50') / 100;
$img_opacity_end = $params->get('img_overlay_opacity_end', '100') / 100;

$lazy_image = ($params->get('vm_lazy_images', 1)) ? 'loading="lazy"' : '';

$module_id = $params->get('module_id', 'visual_menu_module_bs4');



//normal or automatic mode
$mode = $params->get('mode', 'normal');
$idbase = null;
switch ($mode) {
    case 'automatic':
        $option = $jinput->get('option', '', 'WORD');
        $view = $jinput->get('view', '', 'WORD');
        if ($option === 'com_content') {
            switch ($view) {
                case 'category':
                    $idbase = $jinput->get('id', '', 'INT');
                    break;
                
                case 'categories':
                    $idbase = $jinput->get('id', '', 'INT');
                    break;
                
                case 'article':
                    $idbase =  $jinput->get('id', '', 'INT');
                    break;
            }
        }
        break;
    case 'normal':
    case 'api':
        $idbase = $module->id;
    default:
        $idbase = $params->get('catid');
        break;
}




if ($mode == 'api') {
    $cacheid = md5(serialize(array($idbase, $module->module)));

    $cacheparams = new stdClass;
    $cacheparams->cachemode = 'id';
    $cacheparams->class = 'ModVisualMenuHelper';
    $cacheparams->method = 'getApiList';
    $cacheparams->methodparams = $params;
    $cacheparams->modeparams = $cacheid;

    //$list = ModuleHelper::moduleCache ($module, $params, $cacheparams);
} else {
    $cacheid = md5(serialize(array($idbase, $module->module)));
    $cacheparams = new stdClass;
    $cacheparams->cachemode = 'id';
    $cacheparams->class = 'ModVisualMenuHelper';
    $cacheparams->method = 'getList';
    $cacheparams->methodparams = $params;
    $cacheparams->modeparams = $cacheid;

    //$list = ModuleHelper::moduleCache ($module, $params, $cacheparams);

    $list = VisualMenuHelper::getList($params);


}


$visual_menu_bs_masonry_on = $params->get('visual_menu_bs_masonry_on', 0);


if ($visual_menu_bs_masonry_on) {
    $wa->useScript('visuamenu.masonry');
}

//return;
//$list = "lala";

if (!empty($list)) {
    $moduleclass_sfx = htmlspecialchars($params->get('moduleclass_sfx'));
    $item_heading = $params->get('item_heading');

    //BASE STYLE
    if ($params->get('base_css', 1)) {
        $wa->registerStyle(
            'visual_menu_base.css',
            URI::root() . 'media/mod_visual_menu/css/visual_menu_base4x.css',
            [],
            []
        );
        $wa->getAsset('style', 'visual_menu_base.css')->setAttribute('rel', 'lazy-stylesheet');
        $wa->useStyle('visual_menu_base.css');
        VisualMenuDebug::addDebug("LOAD BASE STYLE", 'visual_menu_base PLAIN');
    }



    //CSS STYLE
    //TODO
    /*
    $style = $params->get('style_css','');

    if($style !='' && $style !='-1'){
        
        if($params->get('load_css_async',0)){
            VisualMenuDebug::addDebug("LOAD CUSTOM STYLE",$style.' ASYNC');
            echo '<noscript id="deferred-styles_visualmenu_style"><link rel="stylesheet" type="text/css" href="'.URI::root().'media/mod_visual_menu/styles/'.$style.'.css"/>
            </noscript>';
    
            echo '<script>
                var loadDeferredStyles_visualmenu_style = function() {
                var addStylesNode = document.getElementById("deferred-styles_visualmenu_style");
                var replacement = document.createElement("div");
                replacement.innerHTML = addStylesNode.textContent;
                document.body.appendChild(replacement)
                addStylesNode.parentElement.removeChild(addStylesNode);
                };
                var raf = requestAnimationFrame || mozRequestAnimationFrame ||
                    webkitRequestAnimationFrame || msRequestAnimationFrame;
                if (raf) raf(function() { window.setTimeout(loadDeferredStyles_visualmenu_style, 0); });
                else window.addEventListener(\'load\', loadDeferredStyles_visualmenu_style);
            </script>';

        }else{
            VisualMenuDebug::addDebug("LOAD CUSTOM STYLE",$style.' PLAIN');
            $document->addStyleSheet(URI::root().'media/mod_visual_menu/styles/'.$style.'.css');
        }
    }
    */





    //$document->addStyleSheet(JURI::root().'modules/mod_visual_menu/css/vm_bs4.css');
    $layout = $params->get('layout', 'default');
    if (Version::MAJOR_VERSION >= 4) {
        require ModuleHelper::getLayoutPath('mod_visual_menu', 'bootstrap5');
    } else {
        require ModuleHelper::getLayoutPath('mod_visual_menu', $layout);
    }
}




if ($debug) {

    $html = '';

    $msgs = VisualMenuDebug::getDebug();


    if ($params->get('cache', 0)) {

        $config = $app->getConfig();
        $global_cache = $config->get('caching');

        if ($global_cache) {
            $html .=  $profiler->mark(' with caching');
        } else {
            $html .=  $profiler->mark(' with caching in module, but disabled in config');
        }
    } else {
        $html .=  $profiler->mark(' no caching');
    }




    $html .= '<div class="table-responsive"><table class="table table-striped table-condensed table-bordered "><thead><tr class="alert alert-success">
    <th class="text-center"><i class="fa fa-bug"></i></th>
    <th class="text-center">Debug</th>
    <th class="text-center"><i class="fa fa-clock-o" aria-hidden="true"></i></th>
    </tr></thead>';
    foreach ($msgs  as $key => $value) {

        $html .= '<tr><td>' . $value['func'] . '</td><td>' . $value['msg'] . '</td><td></td></tr>';
    }

    $html .= '</table></div>';

    echo $html;
}
