<?php

/**
 * @package     VisualMenu
 * @subpackage  mod_visual_menu
 *
 * @copyright   (C) 2021 jtotal based on Joomla
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Joomla\Module\VisualMenu\Site\Helper;

use Joomla\CMS\Access\Access;
use Joomla\CMS\Categories\Categories;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Factory;
use Joomla\CMS\Helper\TagsHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\Component\Content\Site\Helper\RouteHelper;
use Joomla\CMS\HTML\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\String\StringHelper;
use stdClass;

\defined('_JEXEC') or die;


abstract class VisualMenuHelper
{


    public static $introtext_remove = '';

    public static function getList(&$params)
    {


        $jinput = Factory::getApplication()->input;
        $option = $jinput->get('option');
        $view = $jinput->get('view');

        //PARAMS
        VisualMenuImageManager::$quality =  $params->get('thumb_quality', 80);
        VisualMenuImageManager::$generate_thumbs =  $params->get('generate_thumbs', 0);
        self::$introtext_remove =  $params->get('introtext_remove', '');



        $app = Factory::getApplication();
        $menu = $app->getMenu();

        /** @var \Joomla\Component\Content\Site\Model\ArticlesModel $model */
        $articles = $app->bootComponent('com_content')->getMVCFactory()->createModel('Articles', 'Site', ['ignore_request' => true]);
        //Set application parameters in model

        $appParams = $app->getParams();
        $articles->setState('params', $appParams);

        // Set the filters based on the module params
        $articles->setState('list.start', 0);
        $articles->setState('list.limit', (int) $params->get('count', 0));
        $articles->setState('filter.published', 1);

        // Access filter
        $access = !ComponentHelper::getParams('com_content')->get('show_noauth');
        $authorised = Access::getAuthorisedViewLevels(Factory::getUser()->get('id'));
        $articles->setState('filter.access', $access);


        $tags = new TagsHelper;
        $item_tags = new TagsHelper;


        //** Prep for Normal or Dynamic Modes
        $mode = $params->get('mode', 'automatic');
        switch ($mode) {
            case 'menu':
    
                $mActive = $menu->getActive();
                $menuItems = $menu->getItems('parent_id', $mActive->id);
                
                if(empty($menuItems)){
                    $nActiveParent = $menu->getItem($mActive->parent_id);
                    $menuItems = $menu->getItems('parent_id', $nActiveParent->id);
                }

                //var_dump($mActive);
 

                //get the menuChildren ids
                $mItems_ids = array();
                $mItemsmenu_data = array();
            
                foreach($menuItems as $mItem) {
                   //echo $mItem->id;
                    
                    $menuItem = $menu->getItem($mItem->id);


                    //ARTICLES
                    if($menuItem->query['option'] == 'com_content' && $menuItem->query['view'] == 'article'){

                        //menu ID --> article ID
                        $mItems_ids[$mItem->id] = $menuItem->query['id'];
                        $mItemsmenu_data[$mItem->id] = 
                            (object) array("id" => $menuItem->query['id'], 
                            "title" => $menuItem->title, 
                            "link" => $menuItem->link, 
                            "route" => $menuItem->route, 
                            "type" => "article"
                        );
                      
                    }
                    //CATEGORY
                    if($menuItem->query['option'] == 'com_content' && $menuItem->query['view'] == 'category'){
                        //var_dump($menuItem);
                        //menu ID --> article ID
                        //$mItems_ids[$mItem->id] = $menuItem->query['id'];
                        $mItemsmenu_data[$mItem->id] = (object) array(
                            "id" => $menuItem->query['id'], 
                            "title" => $menuItem->title, 
                            "link" => $menuItem->link, 
                            "route" => $menuItem->route, 
                            "type" => "category"
                        );
                    }


                  
                }
                //var_dump($mItems_ids);
                if(!empty($mItems_ids)){
                    $articles->setState('filter.article_id', $mItems_ids);
                }
             
        
               // var_dump($mItems_ids);
               //$articles->setState('filter.article_id', array(1346));


                break;
            case 'automatic':

                if ($option === 'com_content') {
                    switch ($view) {
                        case 'category':
                            $catids = array($app->input->getInt('id'));
                            
                            $tags->getItemTags('com_content.category', $app->input->getInt('id'));
                            $tags_ids = array();
                            foreach ($tags->itemTags as $tag) {
                                $tags_ids[] = $tag->id;
                            }
         
                            break;
                        case 'categories':
                            $catids = array($app->input->getInt('id'));

                            break;
                        case 'article':
                            if ($params->get('show_on_article_page', 1)) {

                                $article_id = $app->input->getInt('id');
                                $catid = $app->input->getInt('catid');

                                if (!$catid) {
                                    // Get an instance of the generic article model
                                    $article = $app->bootComponent('com_content')
                                        ->getMVCFactory()->createModel('Article', 'Site', ['ignore_request' => true]);

                                    $article->setState('params', $appParams);
                                    $article->setState('filter.published', 1);
                                    $article->setState('article.id', (int) $article_id);
                                    $item = $article->getItem();

                                    $catids = array($item->catid);
                                } else {
                                    $catids = array($catid);
                                }

                                $tags->getItemTags('com_content.article', $app->input->getInt('id'));
                                $tags_ids = array();
                                foreach ($tags->itemTags as $tag) {
                                    $tags_ids[] = $tag->id;
                                }

              

                            } else {
                                // Return right away if show_on_article_page option is off
                                return;
                            }
                            break;

                        case 'featured':
                        default:
                            // Return right away if not on the category or article views
                            return;
                    }
                } else {
                    // Return right away if not on a com_content page
                    return;
                }
                break;
            case 'normal':
            default:
                $catids = $params->get('catid');
                $articles->setState('filter.category_id.include', (bool) $params->get('category_filtering_type', 1));
                break;
        }



        ///////////////////
        //FILTERS
        ///////////////////
        if($mode == 'automatic'){
            $filterMode = $params->get('filter_mode', 'cat');

            ///////////////////
            //FILTER MODE TAG
            ///////////////////

            if($filterMode == 'tag' && !empty($tags_ids)){
                $articles->setState('filter.tag', $tags_ids);               
            }else{

                //********************
                //** Category filter
                //********************
                if ($catids) {

                    //si tiene hijos y el level es mayor q 0
                    if ($params->get('show_child_category_articles', 0) && (int) $params->get('levels', 0) > 0) {
                        // Get an instance of the generic categories model

                        //$categories = JModelLegacy::getInstance('Categories', 'ContentModel', array('ignore_request' => true));
                        $categories = $app->bootComponent('com_content')->getMVCFactory()
                            ->createModel('Categories', 'Site', ['ignore_request' => true]);

                        $categories->setState('params', $appParams);
                        $levels = $params->get('levels', 1) ? $params->get('levels', 1) : 9999;

                        //echo "tiene hijos | level:".$levels." | catids: ".$catids[0]." | count:".count($catids)."<br>";

                        $categories->setState('filter.get_children', $levels);
                        $categories->setState('filter.published', 1);
                        $categories->setState('filter.access', $access);
                        $additional_catids = array();

                        //echo "categorias hijo: ";
                        foreach ($catids as $catid) {

                            $categories->setState('filter.parentId', $catid);
                            $recursive = true;
                            $items = $categories->getItems($recursive);

                            if ($items) {
                                foreach ($items as $category) {
                                    $condition = (($category->level - $categories->getParent()->level) <= $levels);
                                    if ($condition) {
                                        $additional_catids[] = $category->id;
                                        //echo $category->id."  |   ";
                                    }
                                }
                            }
                        }

                        /****** HIDE CATEGORIES ******/
                        $hide_current_cat = $params->get('hide_current', '1');


                        if ($hide_current_cat == "0") {
                            if (count($additional_catids) == 0) {
                                $catids = array_unique(array_merge($catids, $additional_catids));
                            } else {
                                $catids = $additional_catids;
                            }
                        } else {
                            $catids = array_unique(array_merge($catids, $additional_catids));
                        }
                    }

                    //******MULTICAT******/
                    //Valoro qué categoría debería aparecer en caso de que haya multiples
                    $cat_itemid_based = $params->get('cat_itemid_based', '0');

            
                    if ($cat_itemid_based) {

                        if (is_array($catids)) {

                            $current_itemid = $jinput->get('Itemid');

                            $multi_menu = $app->getMenu();
                            $multi_item = $multi_menu->getItem($current_itemid);
                            $multi_link = new Uri($multi_item->link);
                            $multi_view = $multi_link->getVar('view');
                            $multi_id = $multi_link->getVar('id');

                            //es category
                            if ($multi_view == 'category') {
                                $articles->setState('filter.category_id', $multi_id);
                                //es article	
                            } elseif ($multi_view == 'article') {
                                //get category id
                                $db = Factory::getContainer()->get('DatabaseDriver');
                                $db->setQuery('SELECT cat.id FROM #__categories cat RIGHT JOIN #__content cont ON cat.id = cont.catid WHERE cont.id=' . $multi_id);
                                $category_title = $db->loadResult();
                                $articles->setState('filter.category_id', $category_title);

                                //$articles->setState('filter.category_id', $catids);
                                //es cualquier cosa
                            } else {
                                $articles->setState('filter.category_id', $catids);
                            }
                        } else {
                            $articles->setState('filter.category_id', $catids);
                        }


                    } else {
                        $articles->setState('filter.category_id', $catids);
                    }
                
                }

            }

        }//end of mode automatic



        
        //********************
        //** Filters
        //********************

        //only if is last category
        $showfeatures = $params->get('show_front', 'show');

        if ($showfeatures == "hide_last_cat") {

            //compruebo si es estoy en el último nivel
            if (count($additional_catids) == 0) {
                //echo "<br>Es el ultimo nivel!!!! ". count($additional_catids)."<br>";
                $articles->setState('filter.featured', 'hide');
            } else {
                //echo "<br>NO es el ultimo nivel!!!! ". count($additional_catids)."<br>";
                $articles->setState('filter.featured', 'only');
            }
        } else {
            $articles->setState('filter.featured', $params->get('show_front', 'show'));
        }


        // Filter by language
        $articles->setState('filter.language', $app->getLanguageFilter());

        ////////////////////////////////
        // Filter by tag
        ////////////////////////////////
        //TODO in xml

        //get the tags 




        

        // Ordering
        $articles->setState('list.ordering', $params->get('article_ordering', 'a.ordering'));
        $articles->setState('list.direction', $params->get('article_ordering_direction', 'ASC'));


        // New Parameters
        //$articles->setState('filter.featured', $params->get('show_front', 'show'));
        //$articles->setState('filter.author_id', $params->get('author_filtering', ""));
        //$articles->setState('filter.author_id.include', $params->get('author_filtering_type', 1));
        //$articles->setState('filter.author_alias', $params->get('created_by_alias', ""));
        //$articles->setState('filter.author_alias.include', $params->get('author_alias_filtering_type', 1));
        $excluded_articles = $params->get('excluded_articles', '');

        if ($excluded_articles) {
            $excluded_articles = explode("\r\n", $excluded_articles);
            $articles->setState('filter.article_id', $excluded_articles);
            $articles->setState('filter.article_id.include', false); // Exclude
        }

        $date_filtering = $params->get('date_filtering', 'off');
        if ($date_filtering !== 'off') {
            $articles->setState('filter.date_filtering', $date_filtering);
            $articles->setState('filter.date_field', $params->get('date_field', 'a.created'));
            $articles->setState('filter.start_date_range', $params->get('start_date_range', '1000-01-01 00:00:00'));
            $articles->setState('filter.end_date_range', $params->get('end_date_range', '9999-12-31 23:59:59'));
            $articles->setState('filter.relative_date', $params->get('relative_date', 30));
        }


        $items = $articles->getItems();



        //********************
        //* Categories as Items
        //********************
        $cats_as_items = $params->get('category_as_item', '0');
        if ($cats_as_items == '1') {
            $cat_ids = join(',', $catids);

            $db = Factory::getDbo();
            $query = $db->getQuery(true);
            $query = "SELECT * from #__categories WHERE parent_id IN($cat_ids) AND published = 1";
            $db->setQuery($query);
            $row = $db->loadAssocList();

            foreach ($row as $key => $cat) {
                //var_dump($row[$key]['params']);
                $catList = new stdClass;
                $catList->iscat = 1;
                $catList->id = $row[$key]['id'];
                $catList->title = $row[$key]['title'];
                $catList->alias = $row[$key]['alias'];
                $catList->catid = $row[$key]['id'];
                $catList->images = $row[$key]['params'];
                $catList->created = $row[$key]['created_time'];
                $catList->fulltext = $row[$key]['description'];
                $catList->alternative_readmore = Text::_('READMORE');
                //$catList->setState('list.ordering', $params->get('article_ordering', 'a.ordering'));
                //$catList->setState('list.direction', $params->get('article_ordering_direction', 'ASC'));

                array_push($items, $catList);
            }
        }


        //********************
        //* Display options
        //********************
        $show_date = $params->get('show_date', 0);
        $show_date_field = $params->get('show_date_field', 'created');
        $show_date_format = $params->get('show_date_format', 'Y-m-d H:i:s');
        $show_category = $params->get('show_category', 0);
        $show_hits = $params->get('show_hits', 0);
        $show_author = $params->get('show_author', 0);
        $show_introtext = $params->get('show_introtext', 0);
        $introtext_limit = $params->get('introtext_limit', 100);


        $forceitemID = $params->get('force_itemid', '0');
        $current_itemid = $jinput->getInt('Itemid');

        $active_article_id = ($option === 'com_content' && $view === 'article') ? $jinput->getInt('id') : 0;



        foreach ($items as $key => $item) {
            $item->readmore = \strlen(trim($item->fulltext));
            $item->slug     = $item->id . ':' . $item->alias;




            ///////////////////
            //check ALL tags
            //remove articles from the list with not ALL tags 
            if($filterMode == 'tag' && !empty($tags_ids)){
                $item_tags->getItemTags('com_content.article', $item->id);

                $itemtags_ids = array();
                foreach ($item_tags->itemTags as $item_tag) {
                    $itemtags_ids[] = $item_tag->id;
                }

                //remove it
                if (!empty(array_diff($tags_ids, $itemtags_ids))) {
                    unset($items[$key]);
                    continue;
                }
            }



            //********************
            //* ACCESS
            //********************
            if ($access || in_array($item->access, $authorised)) {
                // We know that user has the privilege to view the article

             

                if (isset($item->iscat)) {
                    $item->link = Route::_('index.php?option=com_content&view=category&id=' . $item->id);
                } else {
                    $item->catslug = $item->catid ? $item->catid . ':' . $item->category_alias : $item->catid;
                    //var_dump($item->catslug);
                    //article
                    //forceID
                    if ($forceitemID == 1) {
                        //var_dump($current_itemid);
                        //$item->link = Route::_('index.php?option=com_content&view=article&id=' . $item->slug . '&catid=' . $item->catslug . '&Itemid=' . $current_itemid);
                        $item->link = 'index.php?option=com_content&view=article&id=' . $item->slug . '&catid=' . $item->catslug . '&Itemid='.$current_itemid;
                        //$item->link = 'index.php?option=com_content&view=article&id=' . $item->slug . '&catid=' . $item->catslug;
                        
                        //wrong
                        // $item->link = RouteHelper::getArticleRoute($item->id, $item->catid);                  
                        //$item->link = RouteHelper::getArticleRoute($item->id, $item->catid);
                        //$item->link = 'index.php?Itemid=' . $current_itemid;
                        //var_dump($item->link);

                    } else {
                     
                        //$item->link = Route::_(RouteHelper::getArticleRoute($item->slug, $item->catslug));
                        $item->link = 'index.php?option=com_content&view=article&id=' . $item->slug . '&catid=' . $item->catslug . '&Itemid='.$current_itemid;
               
                        //var_dump($item->link);
                    }
                }




            } else {
                // Angie Fixed Routing
                $app    = Factory::getApplication();
                $menu    = $app->getMenu();
                $menuitems    = $menu->getItems('link', 'index.php?option=com_users&view=login');

                if (isset($menuitems[0])) {
                    $Itemid = $menuitems[0]->id;
                } else if ($jinput->getInt('Itemid') > 0) { //use Itemid from requesting page only if there is no existing menu
                    $Itemid = $jinput->getInt('Itemid');
                }

                $item->link = Route::_('index.php?option=com_users&view=login&Itemid=' . $Itemid);
            }


            // Used for styling the active article
            $item->active = $item->id == $active_article_id ? 'active' : '';

            //DATE
            $item->displayDate = '';
            if ($show_date) {
                $date = new Date($item->$show_date_field);
                $item->displayDate = $date->format($show_date_format);
            }

            //CAT
            if ($item->catid) {
                $item->displayCategoryLink = Route::_(RouteHelper::getCategoryRoute($item->catid));

                $item->displayCategoryTitle = $show_category
                    ? '<a href="' . $item->displayCategoryLink . '">' . $item->category_title . '</a>'
                    : '';
            } else {
                $item->displayCategoryTitle = $show_category ? $item->category_title : '';
            }

            $item->displayHits = $show_hits ? $item->hits : '';
            $item->displayAuthorName = $show_author ? $item->author : '';


            $item = self::getVMImages($item, $params);


            //MODIFIERS
            if ($show_introtext) {

                $introtext_modified =  self::_cleanIntrotext($item->introtext);
                $item->displayIntrotext = HTMLHelper::_('string.truncateComplex', $introtext_modified, $params->get('introtext_max', 200, 'INT'));
            }
        }

        //var_dump($items);
        //ORDER BY MENU
        $newOrder = array();
        if($mode == 'menu'){

            //var_dump($items);
            //order
            foreach ($mItemsmenu_data as $itemid => $mData) {
                
          
                if($mData->type == 'category'){

                    $categories = Categories::getInstance('content');
                    // The above replaces the older format $categories = JCategories::getInstance($extension);
                    $currentCat = $categories->get($mData->id);

                    $cat_images =  $currentCat->getParams()->get('image');
                    //TODO refactorizar todo esto
                    VisualMenuImageManager::$generate_thumbs = $params->get('generate_thumbs', 0);
                    $thumbs_path = $params->get('thumbs_path', 'dx_visualmenu_thumbs');
                    $thumb_width = $params->get('thumb_width', 100);
        
                    $webp = $params->get('vm_support_webp', '0');
                    $tech = $params->get('vm_generate_tech', '0');
                    //creo el directorio si no esta creado
                    $ruta_thumb = JPATH_BASE . '/cache/' . $thumbs_path;

                   if(VisualMenuImageManager::$generate_thumbs) {
                        $catFinalImage = VisualMenuImageManager::_createThumbs($cat_images, $ruta_thumb, $thumbs_path, $thumb_width, $webp, $tech);
                    } else {
                        $catFinalImage = $cat_images;
                    } 

          

                    $newOrder[] = (object) array(
                        'title' => $mData->title,
                        'link' => $mData->link,
                        'displayImage' => $catFinalImage,
                    );
                }

                if($mData->type == 'article'){
                    foreach ($items as $key2 => $item) {

     

                        if($item->id == $mData->id){

                            //var_dump($item->link);
                            //var_dump($mData);
                              
                            $item->link = $mData->route;
                            //$item->link = $item->link.'&Itemid='.$itemid;
    
                            if($params->get('title_from_menu', '0')){
                                $item->title = $mItemsmenu_data[$itemid]->title;
                            }
    
                            $newOrder[] = $item;
                        }
                        // var_dump($mItemsmenu_data[$key]);
    
                    }
                }



            }
            
            $items = $newOrder;


        }






        return $items;
    }


    public static function _cleanIntrotext($introtext, $max = 200)
    {
        $introtext = str_replace('<p>', ' ', $introtext);
        $introtext = str_replace('</p>', ' ', $introtext);
        $introtext = strip_tags($introtext, '<a><strong>');
        //$introtext = strip_tags($introtext);
        $introtext = trim($introtext);


        //custom remove from introtext
        if (self::$introtext_remove != '') {
            $pieces = array_map('trim', explode(',', self::$introtext_remove));

            foreach ($pieces as $key => $piece) {
                $regex        = '/' . $piece . '(.*?)}/i';
                preg_match_all($regex, $introtext, $matches, PREG_SET_ORDER);

                foreach ($matches as $match) {
                    $introtext = str_replace($match[0], '', $introtext);
                    VisualMenuDebug::addDebug("_cleanIntrotext: loadmoduleid", $match[0] . ' REMOVED');
                }
            }
        }

        return $introtext;
    }

    public static function getVMImages($item, $params){

            //***********************************/
            //*********** IMAGES ****************/
            //***********************************/
            //Thumbs
            VisualMenuImageManager::$generate_thumbs = $params->get('generate_thumbs', 0);
            $thumbs_path = $params->get('thumbs_path', 'dx_visualmenu_thumbs');
            $thumb_width = $params->get('thumb_width', 100);

            $webp = $params->get('vm_support_webp', '0');
            $tech = $params->get('vm_generate_tech', '0');

            if (!VisualMenuImageManager::_checkGD()) VisualMenuImageManager::$generate_thumbs = 0;
  
            //creo el directorio si no esta creado
            $ruta_thumb = JPATH_BASE . '/cache/' . $thumbs_path;

            //var_dump($ruta);
            VisualMenuImageManager::_createPath($ruta_thumb);

            //busco imagen en la configuracion del article (intro image)
            $introimage = json_decode($item->images);

            $no_imagen = false;

            //CHECK IMAGES in INTROIMAGE
            if (!isset($introimage->image_intro)) {
                $no_imagen = true;
            } else {

                if ($introimage && strlen($introimage->image_intro) != 0) {

                    if (VisualMenuImageManager::$generate_thumbs) {
                        $item->displayImage = VisualMenuImageManager::_createThumbs($introimage->image_intro, $ruta_thumb, $thumbs_path, $thumb_width, $webp, $tech);
                    } else {
                        $item->displayImage = $introimage->image_intro;
                    }

                    //$item->displayImage = $introimage->image_intro;
                    $item->displayAlt = $introimage->image_intro_alt;
                    
                } elseif ($introimage && strlen($introimage->image_fulltext) != 0) {

                    //self::_createPath();
                    //self::$generate_thumbs = $params->get('generate_thumbs', 1);
                    //$thumbs_path = $params->get('thumbs_path', 'images/dx_visualmenu_thumbs');
                    //$thumb_width = $params->get('thumb_width', 100);

                    if (VisualMenuImageManager::$generate_thumbs) {
                        $item->displayImage = VisualMenuImageManager::_createThumbs($introimage->image_fulltext, $ruta_thumb, $thumbs_path, $thumb_width, $webp, $tech);
                    } else {
                        $item->displayImage = $introimage->image_fulltext;
                    }

                    $item->displayAlt = $introimage->image_fulltext_alt;
                } elseif ($introimage && isset($introimage->image)) {

                    //cat image
                    if (VisualMenuImageManager::$generate_thumbs) {
                        $item->displayImage = VisualMenuImageManager::_createThumbs($introimage->image, $ruta_thumb, $thumbs_path, $thumb_width, $webp, $tech);
                    } else {
                        $item->displayImage = $introimage->image;
                    }
                    //$item->displayImage = $introimage->image;
                    $item->displayAlt = $introimage->image;
                } else {
                    $no_imagen = true;
                }
            }




            if ($no_imagen) {

                if (!isset($item->introtext)) $item->introtext = '';
                //$item->introtext_prepared = JHtml::_('content.prepare', $item->introtext);
                $item->introtext_prepared = HTMLHelper::_('content.prepare', $item->introtext);

                $imagen = preg_match_all('/<img[^>]+>/i', $item->introtext_prepared, $match);


                if ((bool) $imagen) {
                    //consigo el alt
                    $src = preg_match_all('/(alt|title|src)=("[^"]*")/i', $match[0][0], $match_src);

                    //compruebo la posición del src en el array
                    if ($match_src[1][0] == 'src') {
                        $ruta = $match_src[2][0];
                        $alt = $match_src[2][1];
                    } else {
                        $ruta = $match_src[2][1];
                        $alt = $match_src[2][0];
                    }

                    //$imgEPos = JString::strpos($item->introtext,'"',$imgSPos+5);
                    //$imgEAlt = JString::strpos($item->introtext,'"',$imgSAlt+5);
                    //echo "cosa:".htmlentities(JString::substr($item->introtext, ($imgSPos+5), ($imgEPos-($imgSPos+5))));
                    //Si hay más de una imagen, inserto sólo la primera
                    /*if($imgSPos > 0){
                        $item->displayImage = JString::trim($ruta, '"');
                        $item->displayAlt = JString::substr($item->introtext, ($imgSAlt+5), ($imgEAlt-($imgSAlt+5)));
                    }*/
                    //$item->displayImage = JString::trim($ruta, '"');
                    if (VisualMenuImageManager::$generate_thumbs) {
                        $item->displayImage = VisualMenuImageManager::_createThumbs(StringHelper::trim($ruta, '"'), $ruta_thumb, $thumbs_path, $thumb_width, $webp, $tech);
                    } else {
                        $item->displayImage = StringHelper::trim($ruta, '"');
                    }

                    $item->displayAlt = StringHelper::trim($alt, '"');
                }
                //$item->introtext =  self::_cleanIntrotext($item->introtext,$params->get('introtext_max', 200));
                //$item->displayIntrotext = $show_introtext ? self::truncate($item->introtext, $introtext_limit) : '';
                // added Angie show_unauthorized
                $item->displayReadmore = $item->alternative_readmore;
            }
            return $item;
    }
}
