<?php
/**
 * @package     VisualMenu
 * @subpackage  mod_visual_menu images
 *
 * @copyright   (C) 2021 jtotal based on Joomla
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Joomla\Module\VisualMenu\Site\Helper;

defined('_JEXEC') or die;

use Joomla\Module\VisualMenu\Site\Helper\VisualMenuDebug;

use Imagick as Imagick;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\HTML\HTMLHelper;

abstract class VisualMenuImageManager
{


    public static $generate_thumbs = 0;
	public static $quality = 80;



    /*create thumbs*/
	public static function _createThumbs($introimage_thumb,$ruta_thumb,$thumbs_path,$thumb_width, $webp=0,$tech=0) {
		$resultado = array();

		$introimage_thumb = HTMLHelper::cleanImageURL($introimage_thumb);

		
		//get extension
		$ext_boom  = explode('.', $introimage_thumb->url);
		$ext = strtolower(end($ext_boom));



		if(self::$generate_thumbs){


			//external or internal image
			if(self::check_http_img($introimage_thumb->url)){
				$ruta_img = $introimage_thumb;
			}else{
				$ruta_img = JPATH_BASE.'/'.$introimage_thumb->url;
			}



			//codification 76b61fb94a1f03dc7382c67c69537f54bd4ef4e9
			$thumb_name = sha1($introimage_thumb->url);




			//if webp support is activated
			$existe = false;

			if($webp) $ext = 'webp';

			if (file_exists($ruta_thumb.'/'.$thumb_name.'.'.$ext)) {

				$imageSize = getimagesize($ruta_thumb.'/'.$thumb_name.'.'.$ext);

				$resultado['path'] = 'cache/'.$thumbs_path.'/'.$thumb_name;
				$resultado['ext'] = $ext;
				$resultado['webp'] = false;
				$resultado['size_w'] = $imageSize[0];
				$resultado['size_h'] = $imageSize[1];
				$resultado['size_string'] = $imageSize[3];


				if($webp){
					if (file_exists($ruta_thumb.'/'.$thumb_name.'.webp')) {
						$resultado['webp'] = true;
					}
				}
				$existe = true;
			}



			//CHECK and return (HORAY!!) or GENERATE
			if ($existe) {
				VisualMenuDebug::addDebug("Generate thumb: ",$thumb_name.' exists. Avoid thumb generation');
				return $resultado;
			}else{
				VisualMenuDebug::addDebug("Generate thumb: ",$introimage_thumb->url.' doesnt exist. Generating');
				
				//Imagick
				if($tech){
					if (extension_loaded('imagick')) {
					
					   	self::_createThumb_imagick($ext,$ruta_img,$ruta_thumb, $thumb_name,$thumb_width,$webp);

					//fallback
					}else{
			
						self::_createThumb_raw($ruta_img,$ruta_thumb,$thumb_name,$thumb_width,false, $webp);
					}
				}else{

					self::_createThumb_raw($ruta_img,$ruta_thumb,$thumb_name,$thumb_width,false, $webp);
				}
			}

		

			//RECHECK
			if (file_exists($ruta_thumb.'/'.$thumb_name.'.'.$ext)) {
				
				$imageSize = getimagesize($ruta_thumb.'/'.$thumb_name.'.'.$ext);

				$resultado['path'] = 'cache/'.$thumbs_path.'/'.$thumb_name;
				$resultado['ext'] = $ext;
				$resultado['webp'] = false;
				$resultado['size_w'] = $imageSize[0];
				$resultado['size_h'] = $imageSize[1];
				$resultado['size_string'] = $imageSize[3];
	
				if($webp){
					if (file_exists($ruta_thumb.'/'.$thumb_name.'.webp')) {
						$resultado['webp'] = true;
					}
				}
				$existe = true;
			}

			//CHECK and return (HORAY!!) or... buuuuh!
			if ($existe) {

				VisualMenuDebug::addDebug("Generate thumb (recheck): ",$thumb_name.' exists in recheck');
				return $resultado;
			}else{
				$resultado = false;
			}

		}

		return $resultado;
	}


	/*create path for thumbs*/
	public static function _checkGD() {
		if(!extension_loaded('gd')){
			return false;
		}else{
			return true;
		}
	}

	/*create path for thumbs*/
	public static function _createPath($path) {
	    if (is_dir($path)) return true;
	    $prev_path = substr($path, 0, strrpos($path, '/', -2) + 1 );
	    $return = self::_createPath($prev_path);
	    return ($return && is_writable($prev_path)) ? mkdir($path) : false;
	}

	public static function _createThumb_imagick($ext,$ruta_img,$ruta_thumb,$filename,$imgwidth,$webp){
			
		if(File::exists($ruta_img) === false){
			VisualMenuDebug::addDebug("_createThumb_imagick",$ruta_img.' | not found ');
			return;
		};

		VisualMenuDebug::addDebug("_createThumb_imagick",$ruta_img.' | '.$imgwidth);

		$img = new Imagick($ruta_img); 
		$img->setImageResolution(72,72); 
		$img->resampleImage(72,72,imagick::FILTER_UNDEFINED,1); 
		$img->scaleImage($imgwidth,0); 
		$img->writeImage($ruta_thumb.'/'.$filename.'.'.$ext); 
		VisualMenuDebug::addDebug("_createThumb_imagick: writeImage",$filename.' | '.$ext);
		
		if($webp){
			$img->writeImage($ruta_thumb.'/'.$filename.'.webp'); 
			VisualMenuDebug::addDebug("_createThumb_imagick: writeImage",$filename.' | webp');
		}
		
		$img->destroy(); 
	}


	public static function _createThumb_raw($src,$dest,$thumb_name,$desired_width = false, $desired_height = false, $webp=false){

		VisualMenuDebug::addDebug("_createThumb_RAW",$src.' | '.$dest.' | '.$thumb_name.' | '.$desired_width.' | '.$desired_height.' | '.$webp);
		
		$quality = self::$quality;

		//CHECK IF IMAGEWEB creation is available
		if($webp){
			if(function_exists('imagewebp') === false){
				$webp = 0;
			}
		}

	    /*If no dimenstion for thumbnail given, return false */
	    if (!$desired_height&&!$desired_width){
			VisualMenuDebug::addDebug("RETURN _createThumb_RAW",$desired_height.' | '.$desired_width.' | !$desired_height&&!$desired_width');
			return false;
		}
	    $fparts = pathinfo($src);
	    $ext = strtolower($fparts['extension']);
	    /* if its not an image return false */
	    if (!in_array($ext,array('gif','jpg','png','jpeg','webp'))){
			VisualMenuDebug::addDebug("RETURN _createThumb_RAW",$ext.' |  !in_array($ext,array(gif,jpg,png,jpeg)');
			return false;
		}

		if(self::check_http_img($src) == false){
			
			if(!file_exists($src)){
				VisualMenuDebug::addDebug("RETURN _createThumb_RAW",$src.' |  file exists?)');
				return false;
			}
		}
	   

	    /* read the source image */
	    if ($ext == 'gif'){
	        $resource = imagecreatefromgif($src);
		}else if ($ext == 'png'){
	        $resource = imagecreatefrompng($src);
	    }else if ($ext == 'jpg' || $ext == 'jpeg'){
	        $resource = imagecreatefromjpeg($src);
	    }else if ($ext == 'webp'){
	        $resource = imagecreatefromwebp($src);
	    }
	    $width  = imagesx($resource);
	    $height = imagesy($resource);

	    /* find the "desired height" or "desired width" of this thumbnail, relative to each other, if one of them is not given  */
	    if(!$desired_height) $desired_height = floor($height*($desired_width/$width));
	    if(!$desired_width)  $desired_width  = floor($width*($desired_height/$height));
	  
	    /* create a new, "virtual" image */
	    $virtual_image = imagecreatetruecolor($desired_width,$desired_height);



		switch ($ext)
		{
		    case 'jpg':
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        break;
			case 'webp':
				imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
				break;
		    case 'jpeg':
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        break;

		    case 'png':
		        imagealphablending($virtual_image, FALSE);
		        imagesavealpha($virtual_image, TRUE);
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        break;
		    case 'gif':
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        $background = imagecolorallocate($virtual_image, 0, 0, 0); 
		        imagecolortransparent($virtual_image, $background);
		        break;
		}

	    //better quality
	    //old
	    //imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);

	    $fparts = pathinfo($dest.'/'.$thumb_name);

	    //$ext = strtolower($fparts['extension']);

	    /* if dest is not an image type, default to jpg */
	    if (!in_array($ext,array('gif','jpg','png','jpeg','webp'))) $ext = 'jpg';
	    $dest = $fparts['dirname'].'/'.$fparts['filename'].'.'.$ext;

	    //webp
	    $dest_webp = $fparts['dirname'].'/'.$fparts['filename'].'.webp';
	    
	    if ($ext == 'gif'){
	        imagegif($virtual_image,$dest);
	    }else if ($ext == 'png'){
	        imagepng($virtual_image,$dest,8);
	        if($webp) imagewebp($virtual_image,$dest_webp,$quality);
	    }else if ($ext == 'jpg' || $ext == 'jpeg'){
	        imagejpeg($virtual_image,$dest,$quality);
			if($webp) imagewebp($virtual_image,$dest_webp,$quality);
			//VisualMenuDebug::addDebug("raw: imagewebp",$dest_webp);
	    }else if ($ext == 'webp'){
			imagewebp($virtual_image,$dest_webp,$quality);
			//VisualMenuDebug::addDebug("raw: imagewebp",$dest_webp);
	    }
		VisualMenuDebug::addDebug("_createThumb_raw: writeImage",'image created: '.$dest.' | q: '.$quality);
	    return array(
	        'width'     => $width,
	        'height'    => $height,
	        'new_width' => $desired_width,
	        'new_height'=> $desired_height,
	        'dest'      => $dest,
	        'dest_webp' => $dest_webp
	    );
	}

    public static function check_http_img($introimage_thumb){

		if(substr( $introimage_thumb, 0, 4 ) === "http"){
			return true;
		}else{
			return false;
		}

	}


}